/*---------------------------------------------------------*\
| GigabyteRGBFusion2BlackwellGPUDefinitions.h               |
|                                                           |
|   Definitions for Gigabyte RGB Fusion 2 Blackwell GPU     |
|                                                           |
|   This file is part of the OpenRGB project                |
|   SPDX-License-Identifier: GPL-2.0-only                   |
\*---------------------------------------------------------*/

#pragma once

#define RGB_FUSION_2_BLACKWELL_GPU_NUMBER_OF_ZONES 6
